@@ -11,7 +11,6 @@ This software is provided by the copyright holders and contributors "as is" and
 In no event shall the copyright owner or contributors be liable for any direct, indirect, incidental, special, exemplary or consequential damages (including, but not limited to, procurement of substitute goods or services; loss of use, data, or profits; or business interruption) however caused and on any theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise) arising in any way out of the use of this software, even if advised of the possibility of such damage.
 */
 /* TODO use unzip library */
-/* TODO make perror() actually include the file name. */
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -35,29 +34,32 @@ In no event shall the copyright owner or contributors be liable for any direct,
 static osso_context_t* osso_context;
 static gboolean cmdline = TRUE;
 
-static void error (const char *format, ...)
+static void err(const char *format, ...)
 {
 	va_list arg;
-	char text [200];
+	char text[2049];
 
-	va_start (arg, format);
+	va_start(arg, format);
 	if (cmdline)
 	{
-		fprintf (stderr, "error: ");
-		vfprintf (stderr, format, arg);
-		fprintf (stderr, "\n");
+		fprintf(stderr, "error: ");
+		vfprintf(stderr, format, arg);
+		fprintf(stderr, "\n");
 	}
-	else
-		vsnprintf (text, sizeof (text), format, arg);
-	va_end (arg);
+	else {
+		if (vsnprintf(text, sizeof (text), format, arg) == -1) {
+			strcpy(text, "(error message too long)");
+		}
+	}
+	va_end(arg);
 
 	if (!cmdline)
 		osso_system_note_dialog (osso_context, text, OSSO_GN_ERROR, NULL);
 }
 
-void perror (const char *syscall)
+void perr(const char* syscall, const char* affected_file)
 {
-	error ("%s failed: %s", syscall, strerror (errno));
+	err("%s(\"%s\") failed: %s", syscall, affected_file, strerror(errno));
 }
 
 /** wait_until_child_terminated: waits until a process terminated.
@@ -83,17 +85,17 @@ static int wait_until_child_terminated(const char* zip_file_name, pid_t child_pi
 	} while (!WIFEXITED(status) && !WIFSIGNALED(status));
 	
 	if (waitpid_result == -1) { /* error */
-		perror("waitpid");
+		perr("waitpid", "<\"unzip\" process>");
 		return -1;
 	}
 	
 	if (WIFEXITED(status)) {
 		if (WEXITSTATUS(status) != 0) {
-			error ("could not unzip \"%s\"", zip_file_name);
+			err("could not unzip \"%s\"", zip_file_name);
 			return WEXITSTATUS(status);
 		}
 	} else if (WIFSIGNALED(status)) {
-		error("could not unzip \"%s\": unzip received signal %d", zip_file_name, WTERMSIG(status));
+		err("could not unzip \"%s\": unzip received signal %d", zip_file_name, WTERMSIG(status));
 		return -1;
 	}
 	
@@ -124,13 +126,13 @@ static char** get_zip_content_file_names(const char* zip_file_name)
 	
 	if (!g_spawn_sync(NULL, (char**) argv, NULL, G_SPAWN_STDERR_TO_DEV_NULL, NULL, NULL, &standard_output, NULL, &exit_code, NULL)) {
 		/* unzip printed reason to stderr */
-		error("could not read zip file \"%s\"", zip_file_name);
+		err("could not read zip file \"%s\"", zip_file_name);
 		return NULL;
 	}
 
 	if (standard_output == NULL || 	exit_code != 0) {
 		g_free(standard_output);
-		error ("could not read zip file \"%s\"", zip_file_name);
+		err("could not read zip file \"%s\"", zip_file_name);
 		return NULL;
 	}
 
@@ -215,9 +217,9 @@ static char* prepare_unzip(const char* zip_file_name)
 		/* since it is a messy file, specially create a confinement directory for it */
 		
 		if (mkdir(result, 0777) == -1) {
+			perr("mkdir", result);
 			g_free(result);
 			g_free(source_directory);
-			perror("mkdir");
 			return NULL;
 		}
 		
@@ -225,9 +227,9 @@ static char* prepare_unzip(const char* zip_file_name)
 		/* it is an orderly file, so just check whether folder doesn't exist */
 		
 		if (mkdir(result, 0777) == -1) { /* oops. */
+			perr("mkdir", result);
 			g_free(result);
 			g_free(source_directory);
-			perror("mkdir");
 			return NULL;
 		}
 		
@@ -240,8 +242,8 @@ static char* prepare_unzip(const char* zip_file_name)
 	g_free(source_directory);
 
 	if (chdir(result) == -1) {
+		perr("chdir", result);
 		g_free(result);
-		perror("chdir");
 		return NULL;
 	}
 	
@@ -263,19 +265,19 @@ static void undo_unzip(const char* zip_extraction_directory_name)
 	
 	if (zip_extraction_directory_name[0] == '/' && zip_extraction_directory_name[1] == 0)
 	{
-		error("tried to remove the root directory \"/\". This is a serious program bug. Please report to <danny.milo@scratchpost.org>");
+		err("tried to remove the root directory \"/\". This is a serious program bug. Please report to <danny.milo@scratchpost.org>.");
 		return; /* chicken */
 	}
 	
 	if (!g_spawn_sync(NULL, (char**) argv, NULL, 0, NULL, NULL, NULL, NULL, &exit_code, NULL) || exit_code != 0) {
 		/* unzip printed reason to stderr */
-		error("could not delete directory \"%s\"", zip_extraction_directory_name);
+		err("could not delete directory \"%s\"", zip_extraction_directory_name);
 		/* ignore error... */
 	}
 	
 	/*
 	if (remove(zip_extraction_directory_name) == -1) {
-		perror("remove");
+		perr("remove");
 		ignore error...
 	}*/
 }
@@ -309,7 +311,7 @@ static int unzip_file(const char* zip_file_name, const char* zip_extraction_dire
 	
 	if (!g_spawn_async(NULL, (char**) argv, NULL, G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &child_pid, NULL)) {
 		/* unzip printed reason to stderr */
-		error ("could not unzip \"%s\"", zip_file_name);
+		perr("unzip", zip_file_name);
 		return -1;
 	}
 
@@ -341,7 +343,7 @@ static int unzip_frontend(const char* zip_file_name)
 	
 	if (source_directory[0] != 0) {
 		if (chdir(source_directory) == -1) {
-			perror("chdir");
+			perr("chdir", source_directory);
 			return -1; /* whoops. this. is. bad. */
 		}
 	}
@@ -358,8 +360,8 @@ static int unzip_frontend(const char* zip_file_name)
 	
 	
 	if (chdir(previous_directory) == -1) {
+		perr("chdir", previous_directory);
 		g_free(previous_directory);
-		perror("chdir");
 		chdir("/"); /* whoops. */
 		return -1;
 	}
@@ -440,7 +442,7 @@ int main(int argc, char* argv[])
 	
 	osso_context = osso_initialize("unzip_fm", "1.0", FALSE, NULL);
 	if (osso_context == NULL) {
-		g_error("could not register document handler service");
+		err("could not register document handler service");
 		exit(1);
 	}
 
